@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2004 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2009 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,10 +12,10 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+# USA.
 
-"""Handle delivery bounces.
-"""
+"""Handle delivery bounces."""
 
 import sys
 import time
@@ -113,7 +113,7 @@ class Bouncer:
             # This is the first bounce we've seen from this member
             info = _BounceInfo(member, weight, day,
                                self.bounce_you_are_disabled_warnings)
-            self.setBounceInfo(member, info)
+            # setBounceInfo is now called below after check phase.
             syslog('bounce', '%s: %s bounce score: %s', self.internal_name(),
                    member, info.score)
             # Continue to the check phase below
@@ -128,7 +128,7 @@ class Bouncer:
             # We've already scored any bounces for this day, so ignore it.
             syslog('bounce', '%s: %s already scored a bounce for date %s',
                    self.internal_name(), member,
-                   time.strftime('%d-%b-%Y', day + (0,)*6))
+                   time.strftime('%d-%b-%Y', day + (0,0,0,0,1,0)))
             # Continue to check phase below
         else:
             # See if this member's bounce information is stale.
@@ -145,26 +145,45 @@ class Bouncer:
                 info.score += weight
                 info.date = day
                 syslog('bounce', '%s: %s current bounce score: %s',
-                       member, self.internal_name(), info.score)
+                       self.internal_name(), member, info.score)
             # Continue to the check phase below
         #
         # Now that we've adjusted the bounce score for this bounce, let's
         # check to see if the disable-by-bounce threshold has been reached.
         if info.score >= self.bounce_score_threshold:
-            syslog('bounce', 'sending %s list probe to: %s (score %s >= %s)',
+            if mm_cfg.VERP_PROBES:
+                syslog('bounce',
+                   'sending %s list probe to: %s (score %s >= %s)',
                    self.internal_name(), member, info.score,
                    self.bounce_score_threshold)
-            self.sendProbe(member, msg)
-            info.reset(0, info.date, info.noticesleft)
+                self.sendProbe(member, msg)
+                info.reset(0, info.date, info.noticesleft)
+            else:
+                self.disableBouncingMember(member, info, msg)
+        # We've set/changed bounce info above.  We now need to tell the
+        # MemberAdaptor to set/update it.  We do it here in case the
+        # MemberAdaptor stores bounce info externally to the list object to
+        # be sure updated information is stored, but we have to be sure the
+        # member wasn't removed.
+        if self.isMember(member):
+            self.setBounceInfo(member, info)
 
     def disableBouncingMember(self, member, info, msg):
         # Initialize their confirmation cookie.  If we do it when we get the
         # first bounce, it'll expire by the time we get the disabling bounce.
         cookie = self.pend_new(Pending.RE_ENABLE, self.internal_name(), member)
         info.cookie = cookie
+        # In case the MemberAdaptor stores bounce info externally to
+        # the list, we need to tell it to save the cookie
+        self.setBounceInfo(member, info)
         # Disable them
-        syslog('bounce', '%s: %s disabling due to probe bounce received',
-               self.internal_name(), member)
+        if mm_cfg.VERP_PROBES:
+            syslog('bounce', '%s: %s disabling due to probe bounce received',
+                   self.internal_name(), member)
+        else:
+            syslog('bounce', '%s: %s disabling due to bounce score %s >= %s',
+                   self.internal_name(), member,
+                   info.score, self.bounce_score_threshold)
         self.setDeliveryStatus(member, MemberAdaptor.BYBOUNCE)
         self.sendNextNotification(member)
         if self.bounce_notify_owner_on_disable:
@@ -262,12 +281,17 @@ class Bouncer:
         msg.send(self)
         info.noticesleft -= 1
         info.lastnotice = time.localtime()[:3]
+        # In case the MemberAdaptor stores bounce info externally to
+        # the list, we need to tell it to update
+        self.setBounceInfo(member, info)
 
     def BounceMessage(self, msg, msgdata, e=None):
         # Bounce a message back to the sender, with an error message if
         # provided in the exception argument.
         sender = msg.get_sender()
         subject = msg.get('subject', _('(no subject)'))
+        subject = Utils.oneline(subject,
+                                Utils.GetCharSet(self.preferred_language))
         if e is None:
             notice = _('[No bounce details are available]')
         else:
